      PROGRAM NEB
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (MAXDIM=10000)
      DIMENSION Q(MAXDIM,MAXDIM),E(MAXDIM),G(MAXDIM,MAXDIM),F(MAXDIM)
      DIMENSION TANGNT(MAXDIM),DELTAQ(MAXDIM)
C
C  COMPUTES THE IRC USING A NEB SCHEME AS DEPICTED BY HENKELMAN,
C  UBERUAGA AND JONSSON IN J CHEM PHYS 113, 9901 (2000)
C
C  NVAR    DIMENSION OF THE SYSTEM
C  NPES    POTENTIAL ENERGY SURFACE: 1-WOLFE 2-KARPLUS
C  NVER    NUMBER OF IMAGES OF THE STRING (INCLUDING FIRST AND LAST)
C  NITER   MAXIMUM NUMBER OF ITERATIONS
C  DKMAX   MAXIMUM VALUE OF THE SPRING CONSTANT
C  DKMIN   MINIMUM VALUE OF THE SPRING CONSTANT
C  DELTAT  DISPLACEMENT CONSTANT
C  SYSMAS  MASS OF THE SYSTEM
C  NINT    NUMBER OF INTEGRATION STEPS BEFORE RECOMPUTING FORCES
C  FMIN    MAXIMUM FORCE NORM AT WHICH THE PROCESS STOPS
C  DISINI  INITIAL DISTORTION TO BE APPLIED TO THE INITIAL GUESS CURVE
C --- NOTE: FOR A NON-VARIABLE SPRING CONSTANT STRATEGY, SET DKMIN = 0
      NVAR = 2
      NPES = 1
      NVER = 18
      NITER = 11
      DKMAX = 1.0D+01
      DKMIN = 0.0D-03
C      DKMIN = 0
      DELTAT = 5.0D-02
      SYSMAS = 1.0D00
      NINT = 1
      FMIN = 1.0D-01
      DISINI = 0.006
C OPEN FORCE FILE
      OPEN(UNIT=8,FILE='force.dat')
C MINIMA
      IF (NPES.EQ.1) THEN
        Q(1,1)= -1.174
        Q(2,1)= 1.477
        Q(1,NVER)=1.124
        Q(2,NVER)=-1.486
      ENDIF
      IF (NPES.EQ.2.) THEN
c usual karplus:
c        Q(1,1)=-2.71D0
c        Q(2,1)=0.15D0
c        Q(1,NVER)=2.71D0
c        Q(2,NVER)=-0.15D0
c half karplus with r=0.03:
        Q(1,1)=0.0D0
        Q(2,1)=0.0D0
        Q(1,NVER)=2.85D0
        Q(2,NVER)=-0.15D0
      ENDIF
C INTERMEDIATE VERTEXES
      DO 100 JVAR=1,NVAR
        DELTAQ(JVAR)=(Q(JVAR,NVER)-Q(JVAR,1))/(NVER-1)
 100  CONTINUE
      NCOMPU=0
      DO 10 IVER=1,NVER
        DO 110 JVAR=1,NVAR
          Q(JVAR,IVER)=Q(JVAR,1)+(IVER-1)*DELTAQ(JVAR)
 110    CONTINUE
C ----- SLIGHTLY DISPLACE THE CURVE TO THE LEFT(-) OR RIGHT(+) -----
        Q(1,IVER)=Q(1,IVER)+((IVER-1)*DISINI*(NVER-IVER))
        CALL PESCOM(Q(1,IVER),G(1,IVER),E(IVER),NPES)
        NCOMPU=NCOMPU+1
 10   CONTINUE
C VARIABLE SPRING CONSTANT
      IF (E(NVER).GE.E(1)) THEN
          EREF=E(NVER)
          EMAX=E(1)
      ELSE
          EREF=E(1)
          EMAX=E(NVER)
      ENDIF
C BEGIN LOOP
      DO 999 KITER=1,NITER
        FNMAX=0.0D00
        DO 20 IVER=2,NVER-1
          IF (E(IVER).GT.EMAX) EMAX=E(IVER)
 20     CONTINUE
        DO 30 IVER=2,NVER-1
C SEGMENT TANGENT
          DO 40 JVAR=1,NVAR
             TANGNT(JVAR)=(Q(JVAR,IVER+1)-Q(JVAR,IVER-1))/
     &                   ABS(Q(JVAR,IVER+1)-Q(JVAR,IVER-1))
 40       CONTINUE
          TANORM=SQRT(SCAPRO(TANGNT,TANGNT,NVAR))
C SPRING CONSTANT 
          IF (E(IVER).GT.E(IVER-1)) THEN
             EI=E(IVER)
          ELSE
             EI=E(IVER-1)
          ENDIF
          IF (EI.GT.EREF) THEN
             DKI=DKMAX-DKMIN*((EMAX-EI)/(EMAX-EREF))
          ELSE
             DKI=DKMAX-DKMIN
          ENDIF
C FORCE CALCULATION
          DO 50 JVAR=1,NVAR
             F(JVAR)=(G(JVAR,IVER)-G(JVAR,IVER)*TANGNT(JVAR)/TANORM)+
     &               (DKI*(ABS(Q(JVAR,IVER+1)-Q(JVAR,IVER))-
     &               ABS(Q(JVAR,IVER)-Q(JVAR,IVER-1)))*
     &               TANGNT(JVAR)/TANORM)
 50       CONTINUE
C PRINT INITIAL FORCE VECTOR OF LAST ITERATION
          IF (KITER.EQ.NITER) THEN
             WRITE(8,9000) E(IVER),(Q(JVAR,IVER),JVAR=1,NVAR),
     &                     (F(JVAR),JVAR=1,NVAR)
          ENDIF
C VELOCITY VERLET FOR THE MINIMIZATION. ALSO CALL FOR THE E AND G
          DO 60 INTSTP=1,NINT
             DO 70 JVAR=1,NVAR
                Q(JVAR,IVER)=Q(JVAR,IVER)+F(JVAR)*DELTAT-
     &                       0.5D0*G(JVAR,IVER)*DELTAT**2.0D0
                F(JVAR)=F(JVAR)-DELTAT*G(JVAR,IVER)/(2*SYSMAS)
 70          CONTINUE
             NCOMPU=NCOMPU+1
             CALL PESCOM(Q(1,IVER),G(1,IVER),E(IVER),NPES)
             DO 80 JVAR=1,NVAR
                F(JVAR)=F(JVAR)-DELTAT*G(JVAR,IVER)/(2*SYSMAS)
 80          CONTINUE
 60       CONTINUE
          FNORM=SCAPRO(F,F,NVAR)
          IF (FNORM.GT.FNMAX) FNMAX=FNORM
 30     CONTINUE
C CONVERGENCE CRITERIA
        IF (FNMAX.LE.FMIN) STOP
 999  CONTINUE
C END OF LOOP. PRINT RESULTS
      DO 888 IVER=1,NVER
          WRITE(6,*) E(IVER),(Q(JVAR,IVER),JVAR=1,NVAR)
 888  CONTINUE
      WRITE(6,*) '#TOTAL NUMBER OF E & G COMPUTATIONS:',NCOMPU
      WRITE(6,*) '#TOTAL NUMBER OF ITERATIONS:',KITER
      WRITE(6,*) '#FINAL MAXIMUM FORCE NORM:',FNMAX
      CLOSE(8)
 9000 FORMAT(5(X,F12.8))
      END
C
      SUBROUTINE PESCOM(COORD,GRAD,ENERGY,NPES)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (MAXDIM=10000)
      DIMENSION COORD(MAXDIM),GRAD(MAXDIM)
C SEP #1: MODIFIED 4-WELL
C S. WOLFE, H.B. SCHLEGEL, I.G. CSIZMADIA, F. BERNARDI, JACS 97,
C 2020(1975)
C
      IF (NPES.EQ.1) THEN 
        X=COORD(1)
        Y=COORD(2)
        ENERGY=X**4+Y**4-2*X**2-4*Y**2+X*Y+0.3*X+0.1*Y
        GRAD(1)=4*X**3-4*X+Y+0.3D0
        GRAD(2)=4*Y**3-8*Y+X+0.1D0
      ENDIF
C
C SEP #2: KARPLUS
C E. NERIA, S. FISCHER, M. KARPLUS JCP 105(5), 1902(1996)
C
      IF (NPES.EQ.2) THEN
        R=0.03
        X=COORD(1)
        Y=COORD(2)
        EXPLUS=EXP(-(X+3)**2-Y**2)
        EXMIN=EXP(-(X-3)**2-Y**2)
        ENERGY=R*(X**2+Y**2)**2+X*Y-9*EXMIN-9*EXPLUS
        GRAD(1)=4*R*X**3+4*R*X*Y**2+Y-9*(6-2*X)*EXMIN+9*(6+2*X)*EXPLUS
        GRAD(2)=4*R*Y**3+4*R*Y*X**2+Y+18*Y*EXPLUS+18*Y*EXMIN
      ENDIF
      RETURN
      END
C
      FUNCTION SCAPRO (AVEC,BVEC,N)
C
C COMPUTE THE SCALAR PRODUCT BETWEEN THE VECTORS AVEC AND BVEC
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION AVEC(*), BVEC(*)
      SUM = 0.0D0
      DO 1 I = 1,N
         SUM = SUM + AVEC(I) * BVEC(I)
    1 CONTINUE
      SCAPRO = SUM
      RETURN
      END

